/* $Id: HashedPw.h $ */
/** @file
 * Main - Password Hashing
 */

/*
 * Copyright (C) 2012 Oracle Corporation
 *
 * This file is part of VirtualBox Open Source Edition (OSE), as
 * available from http://www.virtualbox.org. This file is free software;
 * you can redistribute it and/or modify it under the terms of the GNU
 * General Public License (GPL) as published by the Free Software
 * Foundation, in version 2 as it comes in the "COPYING" file of the
 * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
 */

#ifndef ___HashedPw_h___
#define ___HashedPw_h___

#include <iprt/cpp/ministring.h>

bool    VBoxIsPasswordHashed(RTCString const *a_pstrPassword);
void    VBoxHashPassword(RTCString *a_pstrPassword);

#endif

